import os

from pathlib import Path
from configparser import NoSectionError

from django.urls import reverse_lazy

from autosecretkey import AutoSecretKey

from .exceptions import ConfigError


# Build paths inside the project like this: BASE_DIR / ...
BASE_DIR = Path(__file__).parent.parent

ASK_PATH = BASE_DIR / "settings.ini"
ASK = AutoSecretKey(ASK_PATH, template=BASE_DIR / "settings.dist.ini")

try:
    if configpath := ASK.config.get("EXPEPHALON", "ConfigPath", fallback=None):
        ASK_PATH = BASE_DIR / "settings.ini"
        ASK = AutoSecretKey(ASK_PATH)
except NoSectionError:
    raise ConfigError(f"{ASK_PATH} is missing an [EXPEPHALON] section. Have you configured everything?")

SECRET_KEY = ASK.secret_key
ALLOWED_HOSTS = ["*"]

DEBUG = ASK.config.getboolean("EXPEPHALON", "Debug", fallback=False)


# Database
# https://docs.djangoproject.com/en/3.0/ref/settings/#databases

if ASK.config.has_section("MariaDB"):
    DB_SECTION = "MariaDB"
elif ASK.config.has_section("MySQL"):
    DB_SECTION = "MySQL"
else:
    raise ConfigError(f"No database configuration section found in {ASK_PATH}")

if DB_SECTION in ("MariaDB", "MySQL"):
    DATABASES = {
        'default': {
            'ENGINE': 'django.db.backends.mysql',
            'NAME': ASK.config[DB_SECTION]["Database"],
            'USER': ASK.config[DB_SECTION]["Username"],
            'PASSWORD': ASK.config.get(DB_SECTION, "Password", fallback=""),
            'HOST': ASK.config.get(DB_SECTION, "Host", fallback="localhost"),
            'PORT': ASK.config.getint(DB_SECTION, "Port", fallback=3306),
        }
    }


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/3.0/howto/static-files/

STATIC_URL = '/static/'

STATICFILES_DIRS = [
    BASE_DIR / "static",
]

if ASK.config.has_section("S3"):
    DEFAULT_FILE_STORAGE = 'storages.backends.s3boto3.S3Boto3Storage'
    AWS_DEFAULT_ACL = None

    STATICFILES_STORAGE = 'storages.backends.s3boto3.S3Boto3Storage'

    AWS_ACCESS_KEY_ID = ASK.config["S3"]["AccessKey"]
    AWS_SECRET_ACCESS_KEY = ASK.config["S3"]["SecretKey"]
    AWS_STORAGE_BUCKET_NAME = ASK.config["S3"]["Bucket"]
    AWS_S3_ENDPOINT_URL = ASK.config["S3"]["Endpoint"]


# Expephalon modules

TEMPLATES_DIRS = [
    BASE_DIR / "templates", 
]

MODULES_DIRS = [
    BASE_DIR / "modules",
]

EXPEPHALON_FRONTEND = ASK.config.get("EXPEPHALON", "FrontendTemplate", fallback="frontend")
EXPEPHALON_BACKEND = ASK.config.get("EXPEPHALON", "BackendTemplate", fallback="backend")
EXPEPHALON_MAIL = ASK.config.get("EXPEPHALON", "MailTemplate", fallback="mail")

EXPEPHALON_MODULES = []


# Caching

if ASK.config.has_section("MEMCACHED"):
    CACHES = {
        'default': {
            'BACKEND': 'django.core.cache.backends.memcached.MemcachedCache',
            'LOCATION': [ASK.config.get("MEMCACHED", "Location", fallback="127.0.0.1:11211")],
        }
    }


CELERY_TASK_SERIALIZER = "pickle"
CELERY_RESULT_SERIALIZER = "pickle"
CELERY_ACCEPT_CONTENT = ['pickle']
CELERY_RESULT_BACKEND = 'django-db'
CELERY_CACHE_BACKEND = 'django-cache'

if ASK.config.has_section("RABBITMQ"):
    RABBITMQ_USER = ASK.config.get("RABBITMQ", "Username", fallback="guest")
    RABBITMQ_PASS = ASK.config.get("RABBITMQ", "Password", fallback="guest")
    RABBITMQ_HOST = ASK.config.get("RABBITMQ", "Host", fallback="127.0.0.1")
    RABBITMQ_PORT = ASK.config.getint("RABBITMQ", "Port", fallback=5672)
    RABBITMQ_VHOST = ASK.config.get("RABBITMQ", "VHost", fallback="")
    
    CELERY_BROKER_URL = f"amqp://{RABBITMQ_USER}:{RABBITMQ_PASS}@{RABBITMQ_HOST}:{RABBITMQ_PORT}/{RABBITMQ_VHOST}"
else:
    CELERY_BROKER_URL = "memory://"

CELERY_BEAT_SCHEDULER = "django_celery_beat.schedulers:DatabaseScheduler"
CELERY_TASK_RESULT_EXPIRES = 12 * 60 * 60
CELERY_WORKER_HIJACK_ROOT_LOGGER = False


LOG_DIRECTORY = Path(ASK.config.get("EXPEPHALON", "LogDirectory", fallback=BASE_DIR / "log"))
LOGLEVEL = ASK.config.get("EXPEPHALON", "LogLevel", fallback="DEBUG")


# Application definition

INSTALLED_APPS = [
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'polymorphic',
    'phonenumber_field',
    'bootstrap4',
    'core',
    'dbsettings',
    'django_celery_results',
    'django_celery_beat',
    'django_countries',
] + EXPEPHALON_MODULES

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

ROOT_URLCONF = 'expephalon.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [ BASE_DIR / "templates" ],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

WSGI_APPLICATION = 'expephalon.wsgi.application'

DEFAULT_AUTO_FIELD = 'django.db.models.BigAutoField'

# Password validation
# https://docs.djangoproject.com/en/3.0/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]


# Internationalization
# https://docs.djangoproject.com/en/3.0/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True


# Password hasher
# https://docs.djangoproject.com/en/3.0/topics/auth/passwords/#how-django-stores-passwords

PASSWORD_HASHERS = [
    'django.contrib.auth.hashers.Argon2PasswordHasher',
    'django.contrib.auth.hashers.PBKDF2PasswordHasher',
    'django.contrib.auth.hashers.PBKDF2SHA1PasswordHasher',
    'django.contrib.auth.hashers.BCryptSHA256PasswordHasher',
]

# Auth URLs

LOGIN_REDIRECT_URL = reverse_lazy('dashboard')
LOGIN_URL = reverse_lazy('login')
LOGOUT_URL = reverse_lazy('logout')

# Logging

try:
    LOG_DIRECTORY.mkdir(exist_ok=True)
except:
    raise Exception(f"Could not create log directory {LOG_DIRECTORY}, please create it manually and make sure the Expephalon user account has sufficient privileges to write to it.")

try:
    with open(os.path.join(LOG_DIRECTORY, "expephalon.log"), "a") as logfile:
        logfile.write("\n----------\n\nStarting Expephalon...\n")
except:
    raise Exception(f"Unable to write to log file {os.path.join(LOG_DIRECTORY, 'expephalon.log')}, please make sure the Expephalon user account has sufficient privileges to write to the {LOG_DIRECTORY} directory and all files within it.")

LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'formatters': {
        'verbose': {
            'format': '%(levelname)s %(asctime)s %(name)s %(process)d %(thread)d %(message)s'
        },
        'simple': {
            'format': '%(levelname)s %(message)s'
        },
    },
    'handlers': {
        'expephalon': {
            'level': LOGLEVEL,
            'class': 'logging.handlers.TimedRotatingFileHandler',
            'filename': os.path.join(LOG_DIRECTORY, 'expephalon.log'),
            'when': 'midnight',
            'backupCount': 10,
            'formatter': 'verbose',
        },
    },
    'loggers': {
        '': {
            'handlers': ['expephalon'],
            'level': 'DEBUG',
            'propagate': False,
        },
    },
}

